/*
 * file: jv_types.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef JV_TYPES_H
#define JV_TYPES_H

#include <asm/byteorder.h>	/* never ever include kernelheader in userspace apps, do we? */

#include <inttypes.h>
#include <stdlib.h>

#define MAX_STREAM_LEN		256
#define MAX_BUFLEN		4096
#define MAX_CONNECTIONS		10
#define MAX_DATA_LEN		(MAX_BUFLEN - sizeof (struct jvisu_sndvar))
#define MAX_SYMBOL_LEN		256

#define JVISU_STAT_INIT		0x01
#define JVISU_STAT_CHANGED	0x02

#define JVISU_PERM_RW		0x00
#define JVISU_PERM_RO		0x01
#define JVISU_PERM_WO		0x02
#define JVISU_INTERVAL		20	/* time between variable checks in ms */
#define JVISU_PORT_DEFAULT	2202


/* Network Commands are defined by the JVisu Native Protocol, so don't change them */
enum jvisu_cmd {
	cmd_alive		= 0,
	cmd_register_var	= 1,
	cmd_unregister_var	= 2,
	cmd_send_var		= 3,
	cmd_conf_data		= 4,
};


typedef enum {
	jv_double	= 1,
	jv_string	= 2,
	jv_bool		= 3,
	jv_byte		= 4,
	jv_short	= 5,
	jv_int		= 6,
	jv_float	= 7,
	jv_stream	= 8,
	jv_bytearray	= 9,
	jv_shortarray	= 10,
	jv_intarray	= 11,
	jv_floatarray	= 12,
	jv_doublearray	= 13,
} jv_var_t;


/*
 * Byteorder stuff, stolen from linux/include/fs/jffs2.h
 */
typedef struct {
	uint64_t v64;
} __attribute__((packed))  jint64_t;

typedef struct {
	uint32_t v32;
} __attribute__((packed))  jint32_t;

typedef struct {
	uint16_t v16;
} __attribute__((packed)) jint16_t;


#define cpu_to_je16(x) ((jint16_t){__cpu_to_le16(x)})
#define cpu_to_je32(x) ((jint32_t){__cpu_to_le32(x)})
#define cpu_to_je64(x) ((jint64_t){__cpu_to_le64(x)})
#define je16_to_cpu(x) (__le16_to_cpu(x.v16))
#define je32_to_cpu(x) (__le32_to_cpu(x.v32))
#define je64_to_cpu(x) (__le64_to_cpu(x.v64))


/** jvisu command header */
struct jvisu_cmdhdr {
	u_char		cmd;
	u_char		type;
	jint16_t	len;
};


/** jvisu register variable command */
struct jvisu_regvar_cmd {
	struct jvisu_cmdhdr	hdr;
	jint32_t		idx;
	jint32_t		rate;
	char			symbol[0];	
};


/** jvisu stream variable */
struct jvisu_varstream {
	int	len;
	char	data[MAX_STREAM_LEN];
};


/** jvisu send variable command */
struct jvisu_sndvar {
	struct jvisu_cmdhdr	hdr;
	jint32_t		idx;
	union {
		u_char		b[0];
		jint16_t	s[0];
		jint32_t	i[0];
		jint64_t	d[0];
	} data;	
};

/** union to represent the internal variable of the application
 *  when writing 
 */
union jvisu_pv {
	u_char			b[0];
	short			s[0];
	int			i[0];
	float			f[0];
	double			d[0];
	uint16_t		u16[0];
	uint32_t		u32[0];
	uint64_t		u64[0];
};

/** jvisu generic telegram */
struct jvisu_sndgen {
	struct jvisu_cmdhdr	hdr;
	u_char			data[0];
};

/** socket structure */
struct jvisu_sock {
	int			sd;
	int			status;
	unsigned int		bufpoi;
	unsigned int		cmdlen;
	u_char			buf[MAX_BUFLEN];
};

/** jvisu registered variable */
struct jvisu_var {
	struct jvisu_sock	*sd;		/* socket descriptor */
	int			type;		/* changed by jvisu  */
	jv_var_t 		inttype;	/* internal type     */
	int			permissions;
	void 			(*write_done) (int arg);
	int			arg;
	int			idx;
	int			rate;
	int			elapsed;
	int			status;
	int			arraylen;
	void			*data;
	u_char			bitmask;
/* 	u_char			dummy[3]; */
	union {
		u_char		b;
		uint16_t	s;
		uint32_t	i;
		float		f;
		double		d;
		u_char		*bp;
		uint16_t	*sp;
		uint32_t	*ip;
		float		*fp;
		double		*dp;
	} cmp;
};

/** jvisu user supplied functions */
struct jvisu_ops {
	int 	(*get_symbol)	(struct jvisu_var	*var,
				 char			*symbol);

	int	(*save_configs)	(int			typ,
				 char			*data);
};


/* struct pv_engine; */

struct jv_server {
	struct pv_engine	*pv_engine;

	/* JVisu */
	int			jv_port;			/* TCP port for JVisu; default=2202 */
	struct jvisu_sock	jv_active_sock[MAX_CONNECTIONS + 1];
	int			jv_connections; 
	fd_set			jv_read_fdset;
	fd_set			jv_write_fdset;
	ptxlist			*jv_varlist; 
	int			jv_registeredvars;		/* FIXME: replace */
	struct timespec		jv_tact;

	/* more - JVisu */
	pthread_t		jvsocket_thread;
	int			jvisu_stop;			/* set 1 to stop JVisu server       */
};


#endif	/* !JV_TYPES_H */
